package me.protocol_client.utils;

import java.awt.Color;

import me.protocol_client.Protocol;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.OpenGlHelper;
import net.minecraft.client.shader.Framebuffer;
import net.minecraft.entity.EntityLivingBase;

import org.lwjgl.opengl.EXTFramebufferObject;
import org.lwjgl.opengl.EXTPackedDepthStencil;
import org.lwjgl.opengl.GL11;

public class OutlineUtilsThanksHex {

	public static void renderOne() {
		checkSetupFBO();
		GL11.glPushAttrib(GL11.GL_ALL_ATTRIB_BITS);
		GL11.glDisable(GL11.GL_ALPHA_TEST);
		GL11.glDisable(GL11.GL_TEXTURE_2D);
		GL11.glDisable(GL11.GL_LIGHTING);
		GL11.glEnable(GL11.GL_BLEND);
		GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);
		GL11.glLineWidth(Protocol.esp.linewidth.getValue());
		GL11.glEnable(GL11.GL_LINE_SMOOTH);
		GL11.glEnable(GL11.GL_STENCIL_TEST);
		GL11.glClear(GL11.GL_STENCIL_BUFFER_BIT);
		GL11.glClearStencil(0xF);
		GL11.glStencilFunc(GL11.GL_NEVER, 1, 0xF);
		GL11.glStencilOp(GL11.GL_REPLACE, GL11.GL_REPLACE, GL11.GL_REPLACE);
		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_LINE);
	}

	public static void renderTwo() {
		GL11.glStencilFunc(GL11.GL_NEVER, 0, 0xF);
		GL11.glStencilOp(GL11.GL_REPLACE, GL11.GL_REPLACE, GL11.GL_REPLACE);
		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_FILL);
	}

	public static void renderThree() {
		GL11.glStencilFunc(GL11.GL_EQUAL, 1, 0xF);
		GL11.glStencilOp(GL11.GL_KEEP, GL11.GL_KEEP, GL11.GL_KEEP);
		GL11.glPolygonMode(GL11.GL_FRONT_AND_BACK, GL11.GL_LINE);
	}

	public static void renderFour(EntityLivingBase entity) {
		if (entity == null) {
			GL11.glColor3f(1, 0.5f, 0);
		} else {
			getColors(entity);
		}
		GL11.glDepthMask(false);
		GL11.glDisable(GL11.GL_DEPTH_TEST);
		OpenGlHelper.setLightmapTextureCoords(OpenGlHelper.lightmapTexUnit, 240.0F, 240.0F);
	}

	public static void renderFour(Boolean echest) {
		if (!echest) {
			GL11.glColor3f(1, 0.7f, 0);
		} else {
			GL11.glColor3f(1, 0, 1);
		}
		GL11.glDepthMask(false);
		GL11.glDisable(GL11.GL_DEPTH_TEST);
		GL11.glEnable(GL11.GL_POLYGON_OFFSET_LINE);
		GL11.glPolygonOffset(1.0F, -2000000F);
		OpenGlHelper.setLightmapTextureCoords(OpenGlHelper.lightmapTexUnit, 240.0F, 240.0F);
	}

	public static void getColors(EntityLivingBase entity) {
		float red = 1;
		float green = 1f;
		float blue = 1F;
		float red2 = 0F;
		float blue2 = 1F;
		float green2 = 1F;
		float ired = 1F;
		float igreen = 0.5f;
		float iblue = 0f;
		if (entity.isInvisible() && !Protocol.getFriendManager().isFriend(entity.getName())) {
			GL11.glColor3f(ired, igreen, iblue);
			return;
		}

		if (Protocol.getFriendManager().isFriend(entity.getName())) {
			GL11.glColor3f(red2, green2, blue2);
			return;
		} else {
			if (!entity.isPlayerSleeping() && !entity.isSneaking() && !entity.isInvisible() && entity.hurtTime == 0) {
				GL11.glColor3f(red, green, blue);
				return;
			}
			if (!entity.isPlayerSleeping() && !entity.isSneaking() && !entity.isInvisible() && entity.hurtTime > 0 || entity.isDead) {
				GL11.glColor3f(1, 0, 0);
				return;
			}
			if (entity.isSneaking() && !entity.isInvisible()) {
				GL11.glColor3f(1F, 0.4F, 0.4F);
				return;
			}
		}
	}

	public static void renderFive() {
		GL11.glPolygonOffset(1.0F, 2000000F);
		GL11.glDisable(GL11.GL_POLYGON_OFFSET_LINE);
		GL11.glEnable(GL11.GL_DEPTH_TEST);
		GL11.glDepthMask(true);
		GL11.glDisable(GL11.GL_STENCIL_TEST);
		GL11.glDisable(GL11.GL_LINE_SMOOTH);
		GL11.glHint(GL11.GL_LINE_SMOOTH_HINT, GL11.GL_DONT_CARE);
		GL11.glEnable(GL11.GL_BLEND);
		GL11.glEnable(GL11.GL_LIGHTING);
		GL11.glEnable(GL11.GL_TEXTURE_2D);
		GL11.glEnable(GL11.GL_ALPHA_TEST);
		GL11.glPopAttrib();
	}

	public static void setColor(Color c) {
		GL11.glColor4d(c.getRed() / 255f, c.getGreen() / 255f, c.getBlue() / 255f, c.getAlpha() / 255f);
	}

	// TheCyberBrick
	public static void checkSetupFBO() {
		// Gets the FBO of Minecraft
		Framebuffer fbo = Minecraft.getMinecraft().getFramebuffer();
		// Check if FBO isn't null
		if (fbo != null) {
			// Checks if screen has been resized or new FBO has been created
			if (fbo.depthBuffer > -1) {
				// Sets up the FBO with depth and stencil extensions (24/8 bit)
				setupFBO(fbo);
				// Reset the ID to prevent multiple FBO's
				fbo.depthBuffer = -1;
			}
		}
	}

	/**
	 * Sets up the FBO with depth and stencil
	 * 
	 * @param fbo
	 *            Framebuffer
	 */
	public static void setupFBO(Framebuffer fbo) {
		// Deletes old render buffer extensions such as depth
		// Args: Render Buffer ID
		EXTFramebufferObject.glDeleteRenderbuffersEXT(fbo.depthBuffer);
		// Generates a new render buffer ID for the depth and stencil extension
		int stencil_depth_buffer_ID = EXTFramebufferObject.glGenRenderbuffersEXT();
		// Binds new render buffer by ID
		// Args: Target (GL_RENDERBUFFER_EXT), ID
		EXTFramebufferObject.glBindRenderbufferEXT(EXTFramebufferObject.GL_RENDERBUFFER_EXT, stencil_depth_buffer_ID);
		// Adds the depth and stencil extension
		// Args: Target (GL_RENDERBUFFER_EXT), Extension (GL_DEPTH_STENCIL_EXT),
		// Width, Height
		EXTFramebufferObject.glRenderbufferStorageEXT(EXTFramebufferObject.GL_RENDERBUFFER_EXT, EXTPackedDepthStencil.GL_DEPTH_STENCIL_EXT, Minecraft.getMinecraft().displayWidth, Minecraft.getMinecraft().displayHeight);
		// Adds the stencil attachment
		// Args: Target (GL_FRAMEBUFFER_EXT), Attachment
		// (GL_STENCIL_ATTACHMENT_EXT), Target (GL_RENDERBUFFER_EXT), ID
		EXTFramebufferObject.glFramebufferRenderbufferEXT(EXTFramebufferObject.GL_FRAMEBUFFER_EXT, EXTFramebufferObject.GL_STENCIL_ATTACHMENT_EXT, EXTFramebufferObject.GL_RENDERBUFFER_EXT, stencil_depth_buffer_ID);
		// Adds the depth attachment
		// Args: Target (GL_FRAMEBUFFER_EXT), Attachment
		// (GL_DEPTH_ATTACHMENT_EXT), Target (GL_RENDERBUFFER_EXT), ID
		EXTFramebufferObject.glFramebufferRenderbufferEXT(EXTFramebufferObject.GL_FRAMEBUFFER_EXT, EXTFramebufferObject.GL_DEPTH_ATTACHMENT_EXT, EXTFramebufferObject.GL_RENDERBUFFER_EXT, stencil_depth_buffer_ID);
	}

}